@@ -4,10 +4,11 @@ Module for functions relevant to image display:
     - set_viewer_options():
       Define options for use with skrt.better_viewer.BetterViewer.
 """
-
 import matplotlib.pyplot as plt
 
-def set_viewer_options():
+from skrt.core import is_list
+
+def set_viewer_options(to_exclude=None, usetex=False, **kwargs):
     """
     Define options for use with skrt.better_viewer.BetterViewer.
 
@@ -16,6 +17,18 @@ def set_viewer_options():
     to BetterViewer as **kwargs.
 
     All options may be overwritten before calling BetterViewer.
+
+    **Parameter:**
+    to_exclude: list
+        List of keys not to include in the returned dictionary.
+
+    usetex: bool, default=False
+        Indicate whether to use LaTeX for text rendering.  Setting to True
+        when no LaTeX installation is avaiable will generate an error.
+
+    kwargs: dict
+        Key-value pairs with which to update BetterViewer options dictionary
+        created here.
     """
     # Set Matplotlib runtime configuration.
     # For details of Matplotlib configuration, see:
@@ -37,7 +50,7 @@ def set_viewer_options():
     plt.rc("mathtext", fontset="dejavuserif")
 
     # Use TeX/LaTeX for typesetting.  (This requires working TeX/LaTeX installation.)
-    plt.rc("text", usetex=True)
+    plt.rc("text", usetex=usetex)
 
     # For ticks, set label size and direction ("in", "out", "inout").
     plt.rc(("xtick", "ytick"), labelsize=25, direction="out")
@@ -62,8 +75,6 @@ def set_viewer_options():
         "scale_in_mm" : True,
         # Indicate whether ticks should be shown on all sides.
         "ticks_all_sides": True,
-        # Omit title.
-        "title": "",
         # Colour map for images.  For pre-defined colour maps, see:
         # https://matplotlib.org/stable/tutorials/colors/colormaps.html
         # "gray": black to white; "binary": white to black.
@@ -76,4 +87,15 @@ def set_viewer_options():
         "clb_label_kwargs": {"labelpad": 5},
         }
 
+    # Update dictionary of BetterViewer image-display options,
+    # using any values passed as keyword arguments.
+    options.update(kwargs)
+
+    # Remove from dictionary any keys that are to be excluded.
+    if isinstance(to_exclude, str):
+        to_exclude = [to_exclude]
+    if is_list(to_exclude):
+        for key in to_exclude:
+            options.pop(key, None)
+
     return options
