@@ -820,7 +820,7 @@ class Dated(PathData):
         for attribute in ["date", "time", "path"]:
             if getattr(self, attribute) != getattr(other, attribute):
                 return getattr(self, attribute) > getattr(other, attribute)
-            return False
+        return False
 
     def __ge__(self, other):
         '''
@@ -1031,7 +1031,7 @@ def alphanumeric(in_str: str = "") -> List[str]:
     to have alphanumeric (natural) sorting"""
 
     elements = []
-    for substr in re.split("(-*[0-9]+)", in_str):
+    for substr in re.split("(-*[0-9]+)", str(in_str)):
         try:
             element = int(substr)
         except BaseException:
@@ -1040,9 +1040,18 @@ def alphanumeric(in_str: str = "") -> List[str]:
     return elements
 
 
-def fullpath(path=""):
-    """Evaluate full path, expanding '~', environment variables, and
-    symbolic links."""
+def fullpath(path="", pathlib=False):
+    """
+    Evaluate full path, expanding '~', environment variables, and
+    symbolic links.
+
+    path: str/pathlib.Path, default=""
+        Path to be expanded.
+
+    pathlib: bool, default=False
+        If False, return full path as string.  If True, return full path
+        as pathlib.Path object.
+    """
 
     expanded = ""
     if path:
@@ -1050,8 +1059,19 @@ def fullpath(path=""):
         tmp = os.path.expandvars(path.strip())
         tmp = os.path.abspath(os.path.expanduser(tmp))
         expanded = os.path.realpath(tmp)
+        if pathlib:
+            expanded = Path(expanded)
     return expanded
 
+def compress_user(path=""):
+    """If path starts with home directory, replace by '~'"""
+    new_path = Path(fullpath(path))
+    try:
+        new_path = f"~/{new_path.relative_to(new_path.home())}"
+    except ValueError:
+        pass
+    return str(new_path)
+
 def qualified_name(cls=None):
     """
     Return qualified name of a class.
@@ -1811,17 +1831,14 @@ class TicToc:
             """
             Set instance attributes.
 
-            Parameters are as documemnted for
+            Parameters are as documented for
             skrt.core.TicToc.__init__() method.
 
             A parameter value of None leaves the corresponding
             attribute value unaltered.
             """
             if message is not None:
-                if message is True:
-                    self.message = self.default_message
-                else:
-                    self.message = message
+                self.message = message
             if time_format is not None:
                 self.time_format = time_format
             if log_level is not None:
@@ -1834,7 +1851,7 @@ class TicToc:
                 out.append(f"{key}: {value}")
             return "\n".join(out)
 
-    def __init__(self, message=None, time_format=None, log_level=None):
+    def __init__(self, message=None, time_format=".6f", log_level=None):
         """
         Constructor of TicToc singleton class.
 
@@ -1843,11 +1860,12 @@ class TicToc:
         message: str/bool, default=None
             Value to be assigned to skrt.core.TicToc().message,
             which defines the default behaviour for printing
-            elapsed time when calling skrt.core.toc().  A value
-            of True sets skrt.core.TicToc().message to
-            "Time elapsed is ".  In the first call to this method,
-            a value of None initialises skrt.core.TicToc().message
-            to False.  In subsequent calls, a value of None is disregarded.
+            elapsed time when calling skrt.core.toc().  If a value
+            of True is set, the string skrt.core.TicToc().default_message
+            (initialised to "Time elapsed is ") is printed.  In the
+            first call to this method, a value of None initialises
+            skrt.core.TicToc().message to False.  In subsequent calls,
+            a value of None is disregarded.
 
         time_format : str, default=None
             Value to be assigned to skrt.core.TicToc().time_format,
@@ -1929,7 +1947,7 @@ def toc(message=None, time_format=None):
     timer = TicToc()
     # Log error and return None if timer hasn't been started.
     if timer.start is None:
-        timer.error("Timer not started - to start timer, call: tic()")
+        timer.logger.error("Timer not started - to start timer, call: tic()")
         return None
 
     # Obtain time elapsed since relevant call to tic().
