@@ -5,24 +5,27 @@ This module implements atlas-based segmentation, where an atlas is
 a reference image, for which regions of interest (ROIs) have been segmented
 in some way.
 
-In single-atlas segmentation, a single atlas is registered to a target image
-that is to be segmented.  Atlas segmentations are then mapped to the target
-image, using the registration transform.  An ROI may be represented as
-a binary mask or as sets of contour points, resulting in two strategies
-for mapping ROIs:
+In single-atlas segmentation, a single atlas and a target image,
+which is the image to be segmented, are registered to one another.  Atlas
+segmentations are then mapped to the target image, using the registration
+transform.  An ROI may be represented as a binary mask or as sets of contour
+points, resulting in two strategies for mapping ROIs:
 
     - atlas and target are taken as fixed and moving image respectively for
       registration, then ROI contour points are pushed from atlas to target;
-    - atlas and target are taken as moving and fixed image respectively for
+    - target and atlas are taken as fixed and moving image respectively for
       the registration, then ROI binary masks are pulled from atlas to target.
 
 In multi-atlas segmentation, single-atlas segmentation is performed
-multiple times, using a different atlas each time, then the results from
-the individual segmentations are combined in some way.
+multiple times, using a different atlas for each segmentation, then
+the results from the individual segmentations are combined in some way.
 
 The following classes are defined:
-SingleAtlasSegmentation() : class for performing single-atlas segmentation;
-MultiAtlasSegmentation() : class for performing multi-atlas segmentation.
+
+- MultiAtlasSegmentation() : class for performing multi-atlas segmentation.
+- SasTuner() : class for varying parameters in a single-atlas segmentation,
+  comparing results for each set of parameters with reference segmentations.
+- SingleAtlasSegmentation() : class for performing single-atlas segmentation;
 
 The following are utility functions used by SingleAtlasSegmentation()
 and MultiAtlasSegmentation():
@@ -53,13 +56,13 @@ from shutil import rmtree
 import pandas as pd
 
 from skrt.core import (
-        get_dict_permutations, get_logger, get_stat_functions, is_list,
-        Data, Defaults, tic, toc)
-from skrt.image import match_images, Image
+        Data, Defaults, get_dict_permutations, get_logger, get_stat_functions,
+        is_list, tic, toc)
+from skrt.image import Image, match_images
 from skrt.structures import (
-        get_by_slice_methods, get_comparison_metrics, get_consensus_types,
-        StructureSet)
-from skrt.registration import Registration
+        StructureSet,
+        get_by_slice_methods, get_comparison_metrics, get_consensus_types)
+from skrt.registration import Registration, engines
 
 class MultiAtlasSegmentation(Data):
     def __init__(
@@ -94,7 +97,8 @@ class MultiAtlasSegmentation(Data):
             rmtree(self.workdir)
 
         # Define contour-propagation strategies and segmentation steps.
-        self.strategies = get_contour_propagation_strategies()
+        args = [kwargs.get(key, None) for key in ["engine", "engine_dir"]]
+        self.strategies = get_contour_propagation_strategies(*args)
         self.steps = get_segmentation_steps()
 
         # Note that these values are passed to the SingleAtlasSegmentation
@@ -147,7 +151,8 @@ class MultiAtlasSegmentation(Data):
     def segment(self, atlas_ids=None, strategy=None, step=None,
             consensus_types=None, force=False, reg_setup_only=False):
         atlas_ids = get_options(atlas_ids, True, list(self.sass))
-        strategy = get_option(strategy, self.default_strategy, self.strategies)
+        strategy = get_strategy(strategy, self.default_strategy,
+                                self.engine, self.engine_dir)
         steps = get_steps(step)
 
         active_ids = []
@@ -211,7 +216,8 @@ class MultiAtlasSegmentation(Data):
     def get_consensus(self, atlas_ids=None, strategy=None, step=None,
                       reg_step=None, consensus_type=None, force=False):
         atlas_ids = get_options(atlas_ids, True, list(self.sass))
-        strategy = get_option(strategy, self.default_strategy, self.strategies)
+        strategy = get_strategy(strategy, self.default_strategy,
+                                self.engine, self.engine_dir)
         step = get_option(step, self.auto_step, self.steps)
         consensus_type = consensus_type or self.default_consensus_type
         if self.consensus_atlas_ids != set(atlas_ids):
@@ -248,7 +254,8 @@ class MultiAtlasSegmentation(Data):
             consensus_types = [consensus_types]
 
         atlas_ids = get_options(atlas_ids, True, list(self.sass))
-        strategy = get_option(strategy, self.default_strategy, self.strategies)
+        strategy = get_strategy(strategy, self.default_strategy,
+                                self.engine, self.engine_dir)
         step = get_option(step, self.auto_step, self.steps)
         all_reg_steps = self.get_sas(atlas_ids[0]).get_reg_steps(step)
         reg_step = get_option(reg_step, None, all_reg_steps)
@@ -414,10 +421,108 @@ class MultiAtlasSegmentation(Data):
         return scores2
 
 
+class SasTuner(Data):
+
+    def __init__(self, pfiles1_variations=None, pfiles2_variations=None,
+                 keep_segmentations=False, **kwargs):
+
+        self.pfiles1_variations = pfiles1_variations
+        self.pfiles2_variations = pfiles2_variations
+        self.keep_segmentations = keep_segmentations
+        self.kwargs = kwargs
+
+        self.sas_constant_kwargs = {}
+        self.sas_variable_kwargs = {}
+        self.comparison_kwargs = {}
+
+        if kwargs:
+            sas_parameters = list(signature(SingleAtlasSegmentation).parameters)
+            comparison_parameters = list(signature(
+                SingleAtlasSegmentation.get_comparison).parameters)
+            for key, value in kwargs.items():
+                if key in sas_parameters and key not in comparison_parameters:
+                    if isinstance(value, list) and len(value) > 1:
+                        self.sas_variable_kwargs[key] = value
+                    else:
+                        self.sas_constant_kwargs[key] = value
+                else:
+                    self.comparison_kwargs[key] = value
+
+            self.sas_constant_kwargs["auto"] = True
+            self.sas_constant_kwargs["auto_reg_setup_only"] = True
+            self.sas_constant_kwargs["overwrite"] = True
+
+        if pfiles1_variations:
+            reg1_permutations = get_dict_permutations(
+                    {reg_step:
+                     get_dict_permutations(pfiles1_variations[reg_step])
+                     for reg_step in pfiles1_variations})
+        else:
+            reg1_permutations = [{}]
+
+        if pfiles2_variations:
+            reg2_permutations = get_dict_permutations(
+                    {reg_step:
+                     get_dict_permutations(pfiles2_variations[reg_step])
+                     for reg_step in pfiles2_variations})
+        else:
+            reg2_permutations = [{}]
+
+        sas_permutations = get_dict_permutations(self.sas_variable_kwargs)
+
+        self.df = None
+        self.sass = []
+        self.adjustments = []
+
+        for sas_permutation in sas_permutations:
+            sas = SingleAtlasSegmentation(
+                    **self.sas_constant_kwargs, **sas_permutation)
+            strategies = get_options(
+                    sas.auto_strategies, sas.default_strategy, sas.strategies)
+            for reg1_permutation in reg1_permutations:
+                reg1_adjustments = sas.adjust_reg_files(
+                        strategies, sas.steps[0], None, reg1_permutation)
+                for reg2_permutation in reg2_permutations:
+                    reg2_adjustments = sas.adjust_reg_files(
+                            strategies, sas.steps[1],
+                            sas.roi_names, reg2_permutation)
+                    adjustments = {**sas_permutation, **reg1_adjustments,
+                                   **reg2_adjustments}
+
+                    df_comparison = sas.get_comparison(**self.comparison_kwargs)
+
+                    self.adjustments.append(deepcopy(adjustments))
+                    if keep_segmentations:
+                        self.sass.append(sas.clone())
+
+                    if adjustments:
+                        df_permutation = pd.DataFrame(
+                                max(1, df_comparison.shape[0]) * [adjustments])
+                    else:
+                        df_permutation = None
+
+                    if df_permutation is None and df_comparison is None:
+                        continue
+                    elif df_permutation is None:
+                        df_sas = df_comparison
+                    elif df_comparison is None:
+                        df_sas = df_permutation
+                    else:
+                        df_sas = pd.concat(
+                                [df_permutation, df_comparison], axis=1)
+
+                    if self.df is None:
+                        self.df = df_sas
+                    else:
+                        self.df = pd.concat(
+                                [self.df, df_sas], ignore_index=True)
+
+
 class SingleAtlasSegmentation(Data):
     def __init__(
         self, im1=None, im2=None, ss1=None, ss2=None, log_level=None,
         workdir="segmentation_workdir", overwrite=False,
+        engine=None, engine_dir=None,
         auto=False, auto_step=None, auto_strategies=None,
         auto_reg_setup_only=False,
         default_step=-1, default_strategy="pull", roi_names=None,
@@ -453,9 +558,13 @@ class SingleAtlasSegmentation(Data):
         if overwrite and self.workdir.exists():
             rmtree(self.workdir)
 
+        # Set registration engine and software directory.
+        self.engine = engine
+        self.engine_dir = engine_dir
+
         # Define recognised contour-propagation strategies
         # and segmentation steps.
-        self.strategies = get_contour_propagation_strategies()
+        self.strategies = get_contour_propagation_strategies(engine, engine_dir)
         self.steps = get_segmentation_steps()
 
         # Set step and strategies for automatic segmenation,
@@ -521,7 +630,8 @@ class SingleAtlasSegmentation(Data):
 
     def segment(self, strategy=None, step=None, force=False,
                 reg_setup_only=False):
-        strategy = get_option(strategy, self.default_strategy, self.strategies)
+        strategy = get_strategy(strategy, self.default_strategy,
+                                self.engine, self.engine_dir)
         steps = get_steps(step)
 
         if force:
@@ -565,6 +675,8 @@ class SingleAtlasSegmentation(Data):
                             log_level=self.log_level,
                             keep_tmp_dir=self.keep_tmp_dir,
                             overwrite=force,
+                            engine=self.engine,
+                            engine_dir=self.engine_dir,
                             )
                         )
 
@@ -619,6 +731,8 @@ class SingleAtlasSegmentation(Data):
                                 log_level=self.log_level,
                                 keep_tmp_dir=self.keep_tmp_dir,
                                 overwrite=force,
+                                engine=self.engine,
+                                engine_dir=self.engine_dir,
                                 )
                             )
 
@@ -681,7 +795,8 @@ class SingleAtlasSegmentation(Data):
 
     def get_registration(self, strategy=None, step=None, roi_name=None,
             force=False):
-        strategy = get_option(strategy, self.default_strategy, self.strategies)
+        strategy = get_strategy(strategy, self.default_strategy,
+                                self.engine, self.engine_dir)
         step = get_option(step, self.default_step, self.steps)
         self.segment(strategy, step, force, reg_setup_only=True)
 
@@ -700,7 +815,8 @@ class SingleAtlasSegmentation(Data):
 
     def get_segmentation(self, strategy=None, step=None, reg_step=None,
             force=False):
-        strategy = get_option(strategy, self.default_strategy, self.strategies)
+        strategy = get_strategy(strategy, self.default_strategy,
+                                self.engine, self.engine_dir)
         step = get_option(step, self.default_step, self.steps)
         self.segment(strategy, step, force)
         segmentations = self.segmentations[strategy][step]
@@ -877,7 +993,100 @@ class SingleAtlasSegmentation(Data):
         else:
             return (initial_step + list(self.pfiles2))
 
+
+def ensure_any(in_val):
+    return in_val
+
+def ensure_dict(in_val, ensure_type=ensure_any):
+    if isinstance(in_val, dict):
+        return {key: ensure_type(val) for key, val in in_val.items()}
+    elif is_list(in_val):
+        return {idx: ensure_type(val) for idx, val in enumerate(in_val)}
+    else:
+        return {0: in_val}
+
+def ensure_image(im):
+    """Return Image cloned from <im>; return None if <im> is None."""
+    return (im if im is None else Image(im))
+
+def ensure_structure_set(ss):
+    """Return StructureSet cloned from <im>; return None if <im> is None."""
+    return (ss if ss is None else StructureSet(ss))
+
+def get_contour_propagation_strategies(engine=None, engine_dir=None):
+    """
+    Return list of contour propagation strategies for registration engine.
+
+    engine: str, default=None
+        String identifying registration engine, corresponding to
+        a key of the dictionary skrt.registration.engines.
+
+    engine_dir: pathlib.Path/str, default=None
+        Path to directory containing registration-engine software.
+        It's assumed that the registration engine is a key of
+        the dictionary skrt.registration.engines, that the directory
+        path includes this key, and that directory path doesn't
+        include any other keys of skrt.registration.engines.
+    ."""
+    # If engine not known, try to determine engine from path to software.
+    if not engine in engines:
+        for local_engine in engines:
+            if local_engine in str(engine_dir):
+                engine = local_engine
+                break
+
+    # If registration engine not known, use default.
+    if not engine in engines:
+        engine = Defaults().registration_engine
+
+    # Return engine strategies for contour propagation.
+    if engine in engines:
+        if engines[engine].transform_points_implemented == True:
+            return ["pull", "push"]
+        else:
+            return ["pull"]
+
+    return []
+
+def get_fixed_and_moving(im1, im2, strategy):
+    """
+    Order target and atlas images as fixed and moving, depending on strategy.
+
+    **Parameters:**
+
+    im1: skrt.image.Image
+        Target image.
+
+    im2: skrt.image.Image
+        Atlas image.
+
+    strategy: str
+       Strategy to be used in contour propagation - one of "pull", "push".
+    """
+    return (im1, im2) if ("pull" == strategy) else (im2, im1)
+
 def get_option(opt=None, fallback_opt=None, allowed_opts=None):
+    """
+    Apply logic for selection allowed option.
+
+    **Parameters:**
+
+    opt: int/bool/other/None, default=None
+        Specification of option:
+        - if an element of <allowed_opts>, <opt> is returned;
+        - if an integer, element <opt> of <allowed_opts> is returned;
+        - if False, an empty string is returned;
+        - if neither <opt> not <fallback_opt> is an element of <allowed_opts>,
+          an integer, or False, element -1 or <allowed_opts> is returned.
+
+    fallback_opt: int/bool/other/None, default=None
+        Alternative specification of option.  Considered in the same was as
+        <opt>, if the latter isn't an element of <allowed_opts>, an integer,
+        or False.
+
+    allowed_opts: list, default=None
+        List of allowed options.
+    """
 
     if not is_list(allowed_opts) or not len(allowed_opts):
         raise RuntimeError("No allowed options specified")
@@ -919,35 +1128,27 @@ def get_options(opts=None, fallback_opts=None, allowed_opts=None):
 
     return options or [allowed_opts[-1]]
 
-def get_fixed_and_moving(im1, im2, strategy):
-    return (im1, im2) if ("pull" == strategy) else (im2, im1)
-
-def ensure_any(in_val):
-    return in_val
-
-def ensure_image(im):
-    return (im if im is None else Image(im))
-
-def ensure_structure_set(ss):
-    return (ss if ss is None else StructureSet(ss))
-
-def ensure_dict(in_val, ensure_type=ensure_any):
-    if isinstance(in_val, dict):
-        return {key: ensure_type(val) for key, val in in_val.items()}
-    elif is_list(in_val):
-        return {idx: ensure_type(val) for idx, val in enumerate(in_val)}
-    else:
-        return {0: in_val}
-
-def get_contour_propagation_strategies():
-    return ["pull", "push"]
-
 def get_segmentation_steps():
+    """Return list of segmentation steps."""
     return ["global", "local"]
 
 def get_steps(step):
+    """
+    Get list of segmentation steps to run, up to and including <step>.
+
+    **Parameter:**
+
+    step: str/int/list/None
+        Specification of segmentation step(s) to be run:
+        - if None, all steps are to be run;
+        - if a string or integer, the corresponding step is to be run;
+        - if a list, all listed steps are to be run.
+    """
+
+    # Obtain list of all segmentation steps.
     all_steps = get_segmentation_steps()
-    # Make list of steps to run
+
+    # Create list of steps from input specification of step.
     if step is None:
         steps_input = all_steps
     elif not is_list(step):
@@ -955,7 +1156,7 @@ def get_steps(step):
     else:
         steps_input = step
 
-    # Check that all steps exist, and convert numbers to names.
+    # Check that all specified steps exist, and convert numbers to names.
     steps = []
     for step_now in steps_input:
         if isinstance(step_now, str):
@@ -967,6 +1168,8 @@ def get_steps(step):
 
         steps.append(step_now)
 
+    # Create list of step names, up to and including the step(s)
+    # specified in input.
     if steps:
         steps_index = max(
                 [all_steps.index(step_now) for step_now in steps])
@@ -980,99 +1183,13 @@ def get_structure_set_index(ss_index, im):
         return len(im.structure_sets) + ss_index
     return ss_index
 
-
-class SasTuner(Data):
-
-    def __init__(self, pfiles1_variations=None, pfiles2_variations=None,
-                 keep_segmentations=False, **kwargs):
-
-        self.pfiles1_variations = pfiles1_variations
-        self.pfiles2_variations = pfiles2_variations
-        self.keep_segmentations = keep_segmentations
-        self.kwargs = kwargs
-
-        self.sas_constant_kwargs = {}
-        self.sas_variable_kwargs = {}
-        self.comparison_kwargs = {}
-
-        if kwargs:
-            sas_parameters = list(signature(SingleAtlasSegmentation).parameters)
-            comparison_parameters = list(signature(
-                SingleAtlasSegmentation.get_comparison).parameters)
-            for key, value in kwargs.items():
-                if key in sas_parameters and key not in comparison_parameters:
-                    if isinstance(value, list) and len(value) > 1:
-                        self.sas_variable_kwargs[key] = value
-                    else:
-                        self.sas_constant_kwargs[key] = value
-                else:
-                    self.comparison_kwargs[key] = value
-
-            self.sas_constant_kwargs["auto"] = True
-            self.sas_constant_kwargs["auto_reg_setup_only"] = True
-            self.sas_constant_kwargs["overwrite"] = True
-
-        if pfiles1_variations:
-            reg1_permutations = get_dict_permutations(
-                    {reg_step:
-                     get_dict_permutations(pfiles1_variations[reg_step])
-                     for reg_step in pfiles1_variations})
-        else:
-            reg1_permutations = [{}]
-
-        if pfiles2_variations:
-            reg2_permutations = get_dict_permutations(
-                    {reg_step:
-                     get_dict_permutations(pfiles2_variations[reg_step])
-                     for reg_step in pfiles2_variations})
-        else:
-            reg2_permutations = [{}]
-
-        sas_permutations = get_dict_permutations(self.sas_variable_kwargs)
-
-        self.df = None
-        self.sass = []
-        self.adjustments = []
-
-        for sas_permutation in sas_permutations:
-            sas = SingleAtlasSegmentation(
-                    **self.sas_constant_kwargs, **sas_permutation)
-            strategies = get_options(
-                    sas.auto_strategies, sas.default_strategy, sas.strategies)
-            for reg1_permutation in reg1_permutations:
-                reg1_adjustments = sas.adjust_reg_files(
-                        strategies, sas.steps[0], None, reg1_permutation)
-                for reg2_permutation in reg2_permutations:
-                    reg2_adjustments = sas.adjust_reg_files(
-                            strategies, sas.steps[1],
-                            sas.roi_names, reg2_permutation)
-                    adjustments = {**sas_permutation, **reg1_adjustments,
-                                   **reg2_adjustments}
-
-                    df_comparison = sas.get_comparison(**self.comparison_kwargs)
-
-                    self.adjustments.append(deepcopy(adjustments))
-                    if keep_segmentations:
-                        self.sass.append(sas.clone())
-
-                    if adjustments:
-                        df_permutation = pd.DataFrame(
-                                max(1, df_comparison.shape[0]) * [adjustments])
-                    else:
-                        df_permutation = None
-
-                    if df_permutation is None and df_comparison is None:
-                        continue
-                    elif df_permutation is None:
-                        df_sas = df_comparison
-                    elif df_comparison is None:
-                        df_sas = df_permutation
-                    else:
-                        df_sas = pd.concat(
-                                [df_permutation, df_comparison], axis=1)
-
-                    if self.df is None:
-                        self.df = df_sas
-                    else:
-                        self.df = pd.concat(
-                                [self.df, df_sas], ignore_index=True)
+def get_strategy(strategy=None, fallback_strategy=None,
+                 engine=None, engine_dir=None):
+    allowed_strategies = get_contour_propagation_strategies(engine, engine_dir)
+    if isinstance(strategy, str) and strategy not in allowed_strategies:
+        word = "strategy" if 1 == len(allowed_strategies) else "strategies"
+        raise RuntimeError(
+                f"Segmentation strategy '{strategy}' not implemented. "
+                f"Implemented {word} for '{engine}' ('{engine_dir}'): "
+                f"{allowed_strategies}")
+    return get_option(strategy, fallback_strategy, allowed_strategies)
