@@ -939,6 +939,38 @@ def test_foreground_bbox_centre_and_widths():
         assert abs(bb_centre[idx] - centre[idx]) <= 1
         assert abs(bb_widths[idx] - 2 * radius) <= 1
 
+@needs_mahotas
+def test_create_intensity_mask():
+    sim = SyntheticImage((100, 100, 40))
+    sim.add_cube(side_length=5, name="cube", centre=(25, 60, 12), intensity=60)
+    sim.add_sphere(radius=10, name="sphere", centre=(80, 20, 12), intensity=50)
+
+    nx, ny, nz = sim.get_n_voxels()
+    ones = np.ones((ny, nx, nz), dtype=np.uint32)
+    masks = [
+            ((None, None), ones),
+            ((45, 55), ones * (sim.get_data() == 50)),
+            ((55, None), ones * (sim.get_data() == 60)),
+            ((None, 55),  1 - ones * (sim.get_data() == 60)),
+            ]
+
+    for intensities, mask2 in masks:
+        vmin, vmax = intensities
+        mask1 = sim.get_intensity_mask(vmin, vmax).get_data()
+        mask1 = mask1.astype(np.uint32)
+        mask1[mask1 > 0] = 1
+
+        assert mask1.shape == mask2.shape
+        if vmin is None and vmax is None:
+            assert mask1.min() == 1
+        else:
+            assert mask1.min() == 0
+        assert mask1.max() == 1
+        assert mask1.min() == mask2.min()
+        assert mask1.max() == mask2.max()
+        assert mask1.sum() == mask2.sum()
+        assert np.all(mask1 == mask2)
+
 @needs_mahotas
 def test_translation_to_align():
     """Test calculation of translation to align pair of images."""
@@ -1292,9 +1324,12 @@ def test_get_centroid():
     # Create synthetic image, featuring sphere.
     # The maximum intensity for the image is shared by the voxels of the sphere,
     # so the calculated centroid should correspond to the sphere centroid.
-    shape = [100, 60, 80]
-    origin = [-40, 40, -100]
-    idx_centre = [80, 15, 35]
+    #shape = [100, 60, 80]
+    #origin = [-40, 40, -100]
+    #idx_centre = [80, 15, 35]
+    shape = [50, 30, 40]
+    origin = [-20, 20, -50]
+    idx_centre = [40, 8, 17]
     sl_centre = [idx_centre[0] + 1, idx_centre[1] + 1, shape[2] - idx_centre[2]]
     pos_centre = [origin[idx] + idx_centre[idx] for idx in range(3)]
     sim = SyntheticImage(shape, origin=origin)
