@@ -7,9 +7,12 @@ import pytest
 import subprocess
 
 from skrt import Image
+from skrt.core import Defaults
 from skrt.simulation import SyntheticImage
-from skrt.registration import (Registration, get_default_pfiles,
-        get_default_pfiles_dir, read_parameters)
+from skrt.registration import (
+        Registration, RegistrationEngine,
+        add_engine, engines, get_default_pfiles, get_default_pfiles_dir,
+        get_engine_cls, read_parameters, set_elastix_dir, set_engine_dir)
 
 from test_structs import compare_rois
 
@@ -361,16 +364,18 @@ def test_shift_parameters():
 
 def test_get_default_pfiles():
     """Test getting list of default parameter files."""
-
-    from skrt.registration import get_default_pfiles
-    default = get_default_pfiles()
-    assert len(default)
-    assert "MI_Affine.txt" in default
+    defaults = [
+            get_default_pfiles(),
+            get_default_pfiles(basename_only=False),
+            get_default_pfiles(pattern="*Affine*"),
+            ]
+    for items in defaults:
+        assert len(items)
+        assert isinstance(items[0], Path)
+        assert "MI_Affine.txt" in [item.name for item in items]
 
 def test_get_default_pfiles_dir():
     """Test getting list of default parameter files via parent directory."""
-
-    from skrt.registration import get_default_pfiles, get_default_pfiles_dir
     pfiles1 = get_default_pfiles()
     pfiles_dir = get_default_pfiles_dir()
     pfiles2 = list(get_default_pfiles_dir().iterdir())
@@ -380,7 +385,7 @@ def test_get_default_pfiles_dir():
     assert len(pfiles1) == len(pfiles2)
 
     for pfile in pfiles2:
-        assert pfile.name in pfiles1
+        assert pfile in pfiles1
 
 def test_add_default_pfile():
     """Test adding a default parameter file."""
@@ -539,3 +544,63 @@ def test_initial_alignment_using_rois():
         assert(Path(reg.tfiles["initial_alignment"]).exists())
         parameters = read_parameters(reg.tfiles["initial_alignment"])
         assert parameters["TransformParameters"] == translation
+
+def engine_tests(engine):
+    assert engine in engines
+    engine_cls = engines[engine]
+    assert isinstance(engine_cls, type)
+    assert issubclass(engine_cls, RegistrationEngine)
+    assert engine == engine_cls.__name__.lower()
+
+def test_add_engine():
+    n_engine = len(engines)
+    assert "newengine" not in engines
+    @add_engine
+    class NewEngine(RegistrationEngine):
+        pass
+    assert n_engine + 1 == len(engines)
+    assert "newengine" in engines
+
+def test_engines():
+    for engine in ["newengine", "elastix", "niftyreg"]:
+        engine_tests(engine)
+
+def test_default_engine():
+    for engine in [Defaults().registration_engine]:
+        engine_tests(engine)
+
+def test_get_engine_cls():
+    """Test retrieval of registration engine."""
+    for engine in engines:
+        assert get_engine_cls(engine) == engines[engine]
+    assert get_engine_cls() == engines.get(Defaults().registration_engine, None)
+
+def test_set_engine_dir():
+    """Test setting of registration engine's installation directory."""
+
+    # Check that exception is raised if engine software not installed
+    # in specified directory.
+    for engine in engines:
+        with pytest.raises(RuntimeError) as error_info:
+            set_engine_dir(path="unknown", engine=engine)
+        assert ("not implemented" in str(error_info.value)
+                or "executable(s) not found" in str(error_info.value))
+
+    # Check that exception is raised if engine name is unknown.
+    with pytest.raises(RuntimeError) as error_info:
+        set_engine_dir(path="unknown", engine="unknown")
+    assert "engine not known" in str(error_info.value)
+
+
+def test_set_elastix_dir():
+    """Test deprecated function for setting elastix installation directory."""
+    # Check that deprecation warning is issued,
+    # and that exception is raised if elastix software not insalled
+    # in specified directory.
+    with pytest.warns(DeprecationWarning) as warning_info:
+        with pytest.raises(RuntimeError) as error_info:
+            set_elastix_dir(path="unknown")
+            assert ("not implemented" in str(error_info.value)
+                    or "executable(s) not found" in str(error_info.value))
+        assert 1 == len(warning_info)
+        assert "set_elastix_dir() deprecated" in warning_info[0].message.args[0]
